-----------------------------------------------------------------------------------------------
-- Client Lua Script for ResourceReplaceMove
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "GameLib"
require "Window"
require "Unit"


-------------------------------------------------------------------------------------
-- ResourceReplaceMove Module Definition
-----------------------------------------------------------------------------------------------
local ResourceReplaceMove = {} 

 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ResourceReplaceMove:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ResourceReplaceMove:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ResourceReplaceMove OnLoad
-----------------------------------------------------------------------------------------------
function ResourceReplaceMove:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ResourceReplaceMove.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	self.tSaved = {}
	self.loaded = false
	self.moveble = false
	
end


-----------------------------------------------------------------------------------------------
-- ResourceReplaceMove OnDocLoaded
-----------------------------------------------------------------------------------------------
function ResourceReplaceMove:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ResourceReplaceMoveForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		
		Apollo.RegisterSlashCommand("eb", "OnResourceReplaceMoveOn", self)
		Apollo.RegisterEventHandler("VarChange_FrameCount","OnUpdateTimer", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- ResourceReplaceMove Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/rrm"
function ResourceReplaceMove:OnResourceReplaceMoveOn(strCmd, strParam)

	local tParams = {}
	for w in string.gmatch(strParam, "%S*") do
		if w ~= "" then
			if tonumber(w) then
				table.insert(tParams, tonumber(w))
			else
				table.insert(tParams, string.lower(w))
			end
		end
	end
	local wndMainResourceFrameEng = Apollo.FindWindowByName("MainResourceFrame")
	if tParams[1] == "move" then
		if self.moveable then
			wndMainResourceFrameEng:RemoveStyle("Moveable")
			self.moveable = false
			wndMainResourceFrameEng:SetSprite("sprAbility_CapActive")
			Print("Dont move it like you mean it!")
		else
			wndMainResourceFrameEng:AddStyle("Moveable")
			self.moveable = true
			wndMainResourceFrameEng:SetSprite("UI_BK3_Holo_InsetFramed_Darker")
			Print("Move it like you mean it!")
		end
	elseif tParams[1] == "scale" then
		if tParams[2] == nil then return end
		wndMainResourceFrameEng:SetScale(tonumber(tParams[2]))
	end
end

function ResourceReplaceMove:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	local wndMainResourceFrameEng = Apollo.FindWindowByName("MainResourceFrame")
	local l,t,r,b = wndMainResourceFrameEng:GetAnchorOffsets()
	--Print("l"..l.."t"..t.."r"..r.."b"..b)
	local wndMainposition = { l = l, t = t, r = r, b = b }
	local scale = wndMainResourceFrameEng:GetScale()
	local tSaved = {}
	
	tSaved.wndMainposition = wndMainposition
	tSaved.scale = scale
	
    -- simply return this value and Apollo will save the file!
    return tSaved
end

function ResourceReplaceMove:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	self.tSaved = tData
		
end

function ResourceReplaceMove:OnUpdateTimer()
	
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Engineer then
		return
	end	

	if self.loaded then return end
	local wndMainResourceFrameEng = Apollo.FindWindowByName("MainResourceFrame")
	
	if self.tSaved.scale then
		wndMainResourceFrameEng:SetScale(tonumber(self.tSaved.scale))
	else
		wndMainResourceFrameEng:SetScale(1)
	end
	
	if self.tSaved.wndMainposition then
		wndMainResourceFrameEng:SetAnchorOffsets(self.tSaved.wndMainposition.l, self.tSaved.wndMainposition.t, self.tSaved.wndMainposition.r, self.tSaved.wndMainposition.b)
	end
	self.loaded = true
	wndMainResourceFrameEng:RemoveStyle("Moveable")
	self.moveable = false
end	

----------------------
-- ResourceReplaceMoveForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function ResourceReplaceMove:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function ResourceReplaceMove:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- ResourceReplaceMove Instance
-----------------------------------------------------------------------------------------------
local ResourceReplaceMoveInst = ResourceReplaceMove:new()
ResourceReplaceMoveInst:Init()
